using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

namespace SuperSimpleGame
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        Texture2D balloonR, balloonG;

        // Set the coordinates to draw the sprites at. Position is represented as a
        // vector with two components (x and y). Both sprites start at <0,0>.
        Vector2 balloonRPos = new Vector2(0, 0);
        Vector2 balloonGPos = new Vector2(50, 50);

        Vector2 balloonRSpeed = new Vector2(50.0f, 50.0f);
        Vector2 balloonGSpeed = new Vector2(50.0f, 50.0f);
        
        // mouse has been clicked
        bool clicked = false;
        // Position of last mouse click. Set to -1 (which is outside the frame and not clickable).
        int mouseX = -1;
        int mouseY = -1;

        MouseState lastMouseState;

        Random rand = new Random();

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {

            this.IsMouseVisible = true;
            lastMouseState = Mouse.GetState();

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);
            balloonR = Content.Load<Texture2D>("balloon");
            balloonG = Content.Load<Texture2D>("balloonG");

        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (Keyboard.GetState().IsKeyDown(Keys.Escape))
            {
                this.Exit();
            }

            UpdateMouse();

            // Move the sprite around. Note: the position and speed are passed
            // by reference, so that any changes that are made to them in the
            // method UpdateSprite also get made to the corresponding instance variables.
            UpdateSprite(gameTime, balloonR, ref balloonRPos, ref balloonRSpeed);
            UpdateSprite(gameTime, balloonG, ref balloonGPos, ref balloonGSpeed);

            base.Update(gameTime);
        }


        protected void UpdateMouse()
        {
            MouseState currentMouse = Mouse.GetState();

            if (lastMouseState.LeftButton.Equals(ButtonState.Released) &&
                currentMouse.LeftButton.Equals(ButtonState.Pressed))
            {
                clicked = true;
            }
            else
            {
                clicked = false;
            }

            // The mouse x and y positions are returned relative to the
            // upper-left corner of the game window.
            mouseX = currentMouse.X;
            mouseY = currentMouse.Y;

            lastMouseState = currentMouse;
        }


        // Makes the sprites move and bounce off the edges. When they have been clicked on,
        // they "jump" away.
        private void UpdateSprite(GameTime gameTime, Texture2D spriteTexture, ref Vector2 spritePosition, ref Vector2 spriteSpeed)
        {
            // if clicked move the sprite somewhere to the edge of the window
            if (clicked)
            {
                Rectangle spriteRect = new Rectangle((int)spritePosition.X, (int)spritePosition.Y, spriteTexture.Width, spriteTexture.Height);
                if (spriteRect.Contains(mouseX, mouseY))
                {
                    spritePosition.X = (float)(rand.NextDouble() * 2 - 1) * 2 * graphics.GraphicsDevice.Viewport.Width;
                    spritePosition.Y = (float)(rand.NextDouble() * 2 - 1) * 2 * graphics.GraphicsDevice.Viewport.Height;
                }
            }

            // Move the sprite by speed, scaled by elapsed time.
            spritePosition += spriteSpeed * (float)gameTime.ElapsedGameTime.TotalSeconds;

            int MaxX = graphics.GraphicsDevice.Viewport.Width - spriteTexture.Width;
            int MinX = 0;
            int MaxY = graphics.GraphicsDevice.Viewport.Height - spriteTexture.Height;
            int MinY = 0;

            // Check for bounce.
            if (spritePosition.X > MaxX)
            {
                spriteSpeed.X *= -1;
                spritePosition.X = MaxX;
            }

            else if (spritePosition.X < MinX)
            {
                spriteSpeed.X *= -1;
                spritePosition.X = MinX;
            }

            if (spritePosition.Y > MaxY)
            {
                spriteSpeed.Y *= -1;
                spritePosition.Y = MaxY;
            }

            else if (spritePosition.Y < MinY)
            {
                spriteSpeed.Y *= -1;
                spritePosition.Y = MinY;
            }

        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            graphics.GraphicsDevice.Clear(Color.CornflowerBlue);

            // Draw the sprite.
            spriteBatch.Begin(SpriteBlendMode.AlphaBlend);
            spriteBatch.Draw(balloonR, balloonRPos, Color.White);
            spriteBatch.Draw(balloonG, balloonGPos, Color.White);
            spriteBatch.End();


            base.Draw(gameTime);
        }
    }
}
